#!/usr/bin/perl

#use DateTime;
#use DateTime::Duration;
#use DateTime::TimeZone;
use File::Path;
#use XBase;
use strict;
use Time::Local;

#-------------------------------------------------------------------------------
# скрипт для ручного запуска, конвертит логи за сутки
# script to manually start, converts logs for day
# запуск: convert_log.pl LOGPATH
# launch: convert_log.pl LOGPATH
#-------------------------------------------------------------------------------

my $DELTATIMEZONE = 0;

# рабочий каталог, там где скрипты лежат и корневой для создания полученных логов. без конечного слеша.
my $DIR = ($0=~/(.*)[\/].*/,$1);
print "work dir: $DIR\n";

require "$DIR/../common.pm";

# путь исходных логов
my $log_dir = $ARGV[0];
if( !$log_dir )
{
    die "Log dir name not defined!\n";
}
print "source log dir: $log_dir\n";

my $year  = undef;
my $month = undef;
my $day = undef;

if( $log_dir =~ /(\d{4})\/(\d{2})\/(\d{2})$/ )
{
    $year  = $1;
    $month = $2;
    $day = $3;
}
else
{
    die "Incorrect log dir name '$log_dir'!\n";
}

print "process $day.$month.$year...\n";

print "make dest path...\n";
mkpath( "$DIR/$year/$month/" );

print "open files...\n";
my %dst_log_path = undef;
my %FILE_OUT = undef;
for(my $hour=0; $hour<24; $hour++)
{
    $hour =~ s/^(\d{1})$/0$1/;
    $dst_log_path{$hour} = "$DIR/$year/$month/$day\_$hour";
    open $FILE_OUT{$hour}, ">".$dst_log_path{$hour} or die "can't open '".$dst_log_path{$hour}."' on write!\n";
}

opendir (DIR,$log_dir);
foreach my $cur_file (readdir(DIR))
{
    if( $cur_file =~ /^log_\d{2}$/ )  
    {
	processFile( "$log_dir/$cur_file" );	
    }
}
closedir (DIR);

#print "close files && prepare && load...\n";
for(my $hour=0; $hour<24; $hour++)
{
    $hour =~ s/^(\d{1})$/0$1/;
    print "$hour : close file...\n";
    close $FILE_OUT{$hour};
    # препарируем
    print "$hour : prepare file...\n";
    prepare_file( "", $dst_log_path{$hour}, "" );
    # добавляем в обработку
    print "$hour : adding log $year-$month-$day-$hour to loader...\n"; 
    system( "/home/BGBillingServer/data_loader.sh -eload=$year-$month-$day-$hour-3");
}

print "end, total time: ".(time()-$^T)."sec\n";

#-------------------------------------------------------------------------------

sub processFile 
{
    my $file_name = shift;
    open FILE_IN, "<$file_name"
	    or die "Can't open '$file_name' on read!\n";
    
    while(<FILE_IN>)
    {
	# record number	6	nnnnnn	6-значный десятичный порядковый номер
        # a_mx_nbr	2	nn	Номер станции MX абонента А
        # b_mx_nbr	2	nn	Номер станции MX абонента Б
        # date	8	yyyymmdd	Дата начала вызова
        # time	6	hhmmss	Время начала вызова
        # a_subnbr	10	nn..n	Полный номер абонента А (выровненный по левому разряду с пробелами в конце)
        # b_subnbr	20	nn..n	Полный номер абонента Б (выровненный по левому разряду с пробелами в конце)        
        # a_int_type	2	nn	Тип интерфейса абонента А: 00...08 определены, 09...99 зарезервированы
        # b_int_type	2	nn	Тип интерфейса абонента Б: 00...08 определены, 09...99 зарезервированы
        # call_type	3	nnn	Тип вызова: десятичное число от 00 до 34  
            
        # site_a	3	nnn	Номер базовой станции абонента А
        # channel_a	5	nnnnn	Номер канала абонента А
        # site_b	3	nnn	Номер базовой станции абонента Б
        # channel_b	5	nnnnn	Номер канала абонента Б
        
        # b_subs_q_time	5	nnnnn	Время ожидания в очереди к абоненту Б (секунд)
        # tc_wait_time	5	nnnnn	Время ожидания канала связи (секунд)
        # b_subs_answ_time	5	nnnnn	Длительность подачи вызывного сигнала абоненту Б (секунд)
        # call_duration	5	nnnnn	Продолжительность вызова (секунд)
        
        # release_mode	3	nnn	Режим отбоя вызова (секунд)
        # spec_func	3	nnn	Специальные функции: от 0 до 5
	# ch_pul_mode	3	nnn	Режим тарифных импульсов
        # ch_pulses	5	nnnnn	Тарифные импульсы
        # call_id	10	nn.n	Идентификатор вызова (целое число)
        # cr_lf	2		0D0A

#NNNNNNAABBDDDDDDDDTTTTTTAAAAAAAAAABBBBBBBBBBBBBBBBBBBBAABBTTTSSSCCCCCSSSCCCCCTTTTTWWWWWTTTTTDDDDDRRRFFFPPPCCCCCIIIIIIIIII
#1     35352008070900001570749     0063929              0 5  4  3   96255   71    0    0    9    9  0  0  0    0 889738557
#2     35352008070823300870341     70355                0 0  4  0   92  0  19965535    0   18 1805 10  0  0    0 896423075
#11    353520080708235902?255/8101?70336                5 0  4255   73  3  12265535    0    6  369  1  0  2    0 889673020
#133   35352008070901172670431     0067481              0 5  4  3  122255   58    0    0   12   43  0  0  0    0 891835870

	#          NNNNNN         AA         BB   DDDD     DD     DD     TT     TT     TT   ...A   ...B           AA           BB        TTT        SSS        CCCCC        SSS        CCCCC        TTTTT        WWWWW        TTTTT        DDDDD        RRR        FFF        PPP      CCCCC  IIIIIIIIII
	if( /^[\d\x20]{6}[\d\x20]{2}[\d\x20]{2}(\d{4})(\d{2})(\d{2})(\d{2})(\d{2})(\d{2})(.{10})(.{20})([\d\x20]{2})([\d\x20]{2})[\d\x20]{3}[\d\x20]{3}([\d\x20]{5})[\d\x20]{3}([\d\x20]{5})([\d\x20]{5})([\d\x20]{5})([\d\x20]{5})([\d\x20]{5})[\d\x20]{3}[\d\x20]{3}[\d\x20]{3}[\d\x20]{5}[\d\x20]{10}\s{0,2}$/ )
	{
    	    my $date_yyyy = $1;
    	    my $date_mm = $2;
    	    my $date_dd = $3;
    	    my $time_hh = $4;
    	    my $time_mm = $5;
    	    my $time_ss = $6;
    	    my $a_subnbr = $7;
    	    my $b_subnbr = $8;
    	    my $a_int_type = $9;
    	    my $b_int_type = $10;
    	    my $channel_a = $11;
    	    my $channel_b = $12;
    	    my $b_subs_q_time = $13;
            my $tc_wait_time = $14;
    	    my $b_subs_answ_time = $15;
            my $call_duration = $16;

            $a_subnbr =~ s/\s//g;
            $b_subnbr =~ s/\s//g;
            $a_int_type =~ s/\s//g;
            $b_int_type =~ s/\s//g;
            $channel_a =~ s/\s//g;
    	    $channel_b =~ s/\s//g;    	    
            $b_subs_q_time =~ s/\s//g;
            $tc_wait_time =~ s/\s//g;
            $b_subs_answ_time =~ s/\s//g;
            $call_duration =~ s/\s//g;
            if( $b_subs_q_time == 65535)
            {
        	$b_subs_q_time = 0;
            }
            if( $tc_wait_time == 65535)
            {
        	$tc_wait_time = 0;
            }
            if( $b_subs_answ_time == 65535)
            {
        	$b_subs_answ_time = 0;
            }

            my $port_from = $channel_a;             
            my $port_to = $channel_b;
	    if( !$port_from )
	    {
	        $port_from = "UNDEF";
	    }
	    if( !$port_to )
	    {
	        $port_to = "UNDEF";
	    }

	    my $number_from_164 = num_to_164( $a_subnbr );
	    my $number_to_164 = num_to_164( $b_subnbr );
	    
	    #Как только пользователь радиотелефона делает посылку вызова,
	    #вызов регестрируется и момент регистрации записывается в поле time.
	    #Счётчики B_SUBS_Q_TIME, TC_WAIT_TIME и B_SUBS_ANSW_TIME требуется прибавлять
	    #к моменту регистрации вызова, если требуется получить точное время начала соединения.
	    
	    # если a_int_type не равно 5 (т.е. абонент А - поинт MX96) и b_int_type
	    # равно 5 (т.е. абонент Б - абонент ТФОП), то вычитаем (Длительность =
	    # call_duration - b_subs_answ_time),	    
	    # а иначе - не вычитаем (Длительность = call_duration)
	    
	    my $duration = $call_duration;
	    if( $a_int_type!=5 && $b_int_type==5 )
	    {
		$duration -= $b_subs_answ_time;
	    }
	    
	    my $start_time0 = timelocal( $time_ss, $time_mm, $time_hh, $date_dd, $date_mm-1, $date_yyyy );
	    my($st_sec,$st_min,$st_hou,$st_day,$st_mon,$st_yea) = localtime( $start_time0 + $DELTATIMEZONE + $b_subs_q_time + $tc_wait_time + $b_subs_answ_time );
	    $st_sec =~ s/^(\d{1})$/0$1/;
	    $st_min =~ s/^(\d{1})$/0$1/;
	    $st_hou =~ s/^(\d{1})$/0$1/;
	    $st_day =~ s/^(\d{1})$/0$1/;
	    ++$st_mon;
	    $st_mon =~ s/^(\d{1})$/0$1/;
	    $st_yea +=1900;	
	    my $start_time = "$st_day.$st_mon.$st_yea $st_hou:$st_min:$st_sec";
	    
	    my $dst_log_record = "$start_time\t$duration\t$a_subnbr\t$number_from_164\t$b_subnbr\t$number_to_164\t$port_from\t$port_to\t0";
	    #print $FILE_OUT{$st_hou} "$dst_log_record\n";
	    *FILE_OUT1 = $FILE_OUT{$st_hou};
	    print FILE_OUT1 "$dst_log_record\n";

        }
        elsif( /^#(.*)$/ )
    	{
	    print "INFO: $1\n";
    	}
    	else
    	{                
    	    print "ERROR! Incorrect line $_\n";
        }	
    }
    
    close FILE_IN;
}


