#!/usr/bin/perl
use DateTime;
use DateTime::Duration;
use DateTime::TimeZone;
use integer;
use strict;
use File::Path;

my $DIR = "/home/ATS/si2000";
my $log_name = $ARGV[0];
my $timezone = `date "+%z"`;
my $moscow_timezone = "+0400";

require "$DIR/../common.pm";
if( !$log_name )
{
    die "Log name not defined!\n";
}

my $dst_log_path = undef;
my $year  = undef;
my $month = undef;
my $day = undef;
my $hour = undef;
my $index = undef;

if( $log_name =~ /(\d{4})\/(\d{2})\/(\d{2})\/(\d{2})/ )
{
	$year  = $1;
	$month = $2;
	$day = $3;
	$hour = $4;
    
	$dst_log_path = "$DIR/$year/$month/$day"."_"."$hour";

}
else
{
    die "Incorrect log name $log_name!\n";
}
print "dst_log_path=$dst_log_path\n";

mkpath("$DIR/$year/$month/" );  

open FILE_OUT, ">$dst_log_path"		    
		or die "Can't open $dst_log_path on write!\n";

opendir (DIR,"$DIR/source/$log_name");
foreach my $cur_file (readdir(DIR))
{

	if( $cur_file =~ /^log_\d{4,}$/ )  
	{
		processFile( "$DIR/source/$log_name/$cur_file");	
	}
}
closedir (DIR);




sub processFile 
{
	my $file_name = shift;
	#print "processing file $file_name..\n";
	open FILE_IN, "<:bytes:raw", $file_name
			or die "Can't open $file_name on read!\n";	


	my $count = 0;

	#my $DATA = ();

	process_record();

	close FILE_IN;



	sub process_record
	{
		my $data = undef;    
		if( read( FILE_IN, $data, 1 ) == 0 )
		{
			return;
		}

		my @f = unpack( 'C', $data );	    
		$count++;

		# call record		
		if ( $f[0] == 200 )
		{
			# 2 - record len (bin)
			# 8 - call id (bin)
			# 3 - flags (bin)
			# 1 - posled(4bit), rezerv(4bit) (bin)
			# 1 - lac_len(3bit), spis_num_len(5bit) (bin)
			read( FILE_IN, $data, 15 );
			@f = unpack( 'nA[8]CCCCC',  $data );
	
			my $record_len = $f[0];
			my $call_id = $f[1];
			my $flags0 = $f[2];
			my $flags1 = $f[3];
			my $posled = ($f[5]&0b11110000)>>4;
			my $lac_len = ($f[6]&0b11100000)>>5;
			my $spis_num_len = ($f[6]&0b00011111);
	
			my $lac = undef;
			my $spis_num = undef;
	
			my $bytes_lac_len = get_bytes_len( $lac_len );
			my $bytes_spis_num_len = get_bytes_len( $spis_num_len );
	
			if( $lac_len > 0 )
			{
				read( FILE_IN, $data, $bytes_lac_len );
				$lac = get_bcd( $data, $lac_len );
			}
			if( $spis_num_len > 0 )
			{
				read( FILE_IN, $data, $bytes_spis_num_len );
				$spis_num = get_bcd( $data, $spis_num_len );
			}
	
			my $var_len = $record_len - 16 - $bytes_lac_len - $bytes_spis_num_len;
	
			#print "record_len2=$record_len lags0=$flags0 flags1=$flags1 posled=$posled lac_len=$lac_len spis_num_len=$spis_num_len spis_num=$spis_num var_len=$var_len lac=$lac\n";
			
			read( FILE_IN, $data, $var_len );		
			my %fields = get_fields( $data, $var_len );

	
			my $duration = get_duration( $fields{START_TIME_RAW}, $fields{STOP_TIME_RAW} );
			my $number_from = $spis_num;
			if ( $number_from != '')
			{
				$number_from = kill_59 ( $number_from );
				my $number_to = $fields{100};

				my $number_from_164 = num_to_164( $number_from );
				my $number_to_164 = num_to_164( $number_to );	
				my $startDate = $fields{102};
				my $stopDate = $fields{103};
				my $port_from = $fields{113};
				my $port_to = $fields{114};

			
				my $dst_log_record = "$startDate\t$duration\t$number_from\t$number_from_164\t$number_to\t$number_to_164\t$port_from\t$port_to\t0\t$duration";
	

				print FILE_OUT "$dst_log_record\n";
			}
			else	
			{
				#print "skipping empty number_from..\n";		
			}

	
			#exit 0;
		}
		# date time change record
		elsif( $f[0] == 210 )
		{
			read( FILE_IN, $data, 15 );
			print "Date time change found\n";
		}
		# lost records record
		elsif( $f[0] == 211 )
		{	
			read( FILE_IN, $data, 18 );
			print "Lost records\n";
		} 
		# reboot record
		elsif( $f[0] == 212 )
		{
			read( FILE_IN, $data, 11 );
			print "Reboot record\n";
		}
		else 
		{
			die "Record parse error. unknown recordId=$f[0]\n";
		}
	
		process_record();
	}

}




sub get_fields
{
    my %result;

    my $data = shift;
    my $len = shift;
    
    for( my $pos = 0; $pos < $len; )
    {
		my @f = unpack( 'C', substr( $data, $pos, 1 ) );
		
		#print( "pos=$pos code=$f[0]\n" );
		
		# number b 
		if( $f[0] == 100 )
		{
			my @num_len = unpack( 'C', substr( $data, $pos + 1, 1 ) );
			my $bytes_num_len = get_bytes_len( $num_len[0] );
			my $num = get_bcd( substr( $data, $pos + 2, $bytes_num_len ), $num_len[0] );
			
			#print "100=>$num\n";
			
			$result{100} = $num;
			$pos += 2 + $bytes_num_len;
		}
		# peredan vizov
		elsif( $f[0] == 101 )
		{
			my @num_len = unpack( 'C', substr( $data, $pos + 2, 1 ) );
			my $bytes_num_len = get_bytes_len( $num_len[0] );
			my $num = get_bcd( substr( $data, $pos + 3, $bytes_num_len ), $num_len[0] );
			
			#print "101=>$num\n";
			
			$result{101} = $num;
			$pos += 3 + $bytes_num_len; 
		}
		# start time
		# ?todo: flag time type
		elsif( $f[0] == 102 )
		{
			
			my $str = substr( $data, $pos + 1, 7 ) ;
			$result{START_TIME_RAW} = $str;

			my $start_time = get_time( $str );
			
			
			#print "102=>$start_time\n";
			
			$result{102} = $start_time;
			$pos += 9;
		}
		# stop time
		# todo: check success flag
		elsif( $f[0] == 103 )
		{
			my $str = substr( $data, $pos + 1, 7 ) ;					
			$result{STOP_TIME_RAW} = $str;
			my $stop_time = get_time( $str );
		
			#print "103=>$stop_time\n";
			
			$result{103} = $stop_time;
			$pos += 9;
		}
		# tariff imp count		
		elsif( $f[0] == 104 )
		{
			my @count = unpack( 'CS', substr( $data, $pos + 1, 3 ) );
			my $count = ($count[0] << 16) | $count[1];
			
			#print "104=>$count\n";
			
			$result{104} = $count;
			$pos += 4;			
		}
 		# base service
 		# ?todo: check to phone call
 		elsif( $f[0] == 105 )
 		{
 			my @service = unpack( 'CC', substr( $data, $pos + 1, 2 ) );
 			my $service_data = $service[0];
 			my $service_phone = $service[1];
 			
 			#print "105=>$service_data $service_phone\n";
 			
 			$result{105} = "$service_data $service_phone";
 			$pos += 3;
 		}
 		# dop service iniciatora
 		elsif( $f[0] == 106 )
 		{
 			#print "106=>skip\n";
 			$pos += 2;
 		}
 		# dop service u visivaemogo
 		elsif( $f[0] == 107 )
 		{
 			#print "107=>skip\n";
 			$pos += 2;
 		}
 		# administr uslugi abonentom
 		elsif( $f[0] == 108 )
 		{
 			#print "108=>skip\n";
 			$pos += 3;
 		}
 		# posledovatel'nost' vvedennih simvolov
 		elsif( $f[0] == 109 )
 		{
 			my @len = unpack( 'C', substr( $data, $pos + 1, 1 ) );
 			my $bytes_len = get_bytes_len( $len[0] );
 			my $posledov = get_bcd( substr( $data, $pos + 2, $bytes_len ), $len[0] );
 			
 			#print "109=>$posledov\n";
 			
 			$result{109} = $posledov;
 			$pos += 2 + $bytes_len; 			
 		}
 		# out category
 		elsif( $f[0] == 110 )
 		{
 			my @category = unpack( 'C', substr( $data, $pos + 1, 1 ) );
 			
 			#print "110=>$category[0]\n";
 			
 			$result{110} = $category[0];
 			$pos += 2;
 		}
 		# tariff dest
 		elsif( $f[0] == 111 )
 		{
 			#print "111=>skip\n";
 			$pos += 2;
 		}
 		# prichina bezuspeshnogo vizova
 		elsif( $f[0] == 112 )
 		{
 			#print "112=>skip\n";
 			$pos += 2;
 		}
 		# vhod tract
 		elsif( $f[0] == 113 )
 		{
 			my @id = unpack( 'nnCnC', substr( $data, $pos + 1, 8 ) );
 			my $id = "$id[0]:$id[1]:$id[2]:$id[3]:$id[4]";
 			 			
 			#print "113=>$id\n";
 			
 			$result{113} = $id;
 			$pos += 9;
 		}
 		# ishod tract
 		elsif( $f[0] == 114 )
 		{
 			my @id = unpack( 'nnCnC', substr( $data, $pos + 1, 8 ) );
 			my $id = "$id[0]:$id[1]:$id[2]:$id[3]:$id[4]";
 			 			
 			#print "114=>$id\n";
 			
 			$result{114} = $id;
 			$pos += 9;
 		}
 		# dlit vizova
 		elsif( $f[0] == 115 )
 		{
 			my @dlit = unpack( 'N', substr( $data, $pos + 1, 4 ) );
 			
 			#print "115=>$dlit[0]\n";
 			
 			$result{115} = $dlit[0];
 			$pos += 5;
 		}
 		# contr summa
 		elsif( $f[0] == 116 )
 		{
 			#print "116=>skip\n";
 			$pos += 4;
 		}
 		# bisnes gruppa, centrecs gruppa
 		elsif( $f[0] == 117 )
 		{
 			#print "117=>skip\n";
 			$pos += 10;
 		}
 		# cod dostupa k seti
 		elsif( $f[0] == 118 )
 		{
 			my @len = unpack( 'C', substr( $data, $pos + 1, 1 ) );
			
			#print "118=>skip $len[0]\n";
			$pos += $len[0];
 		}
		#orig nomer vizivaushego abonenta
		elsif( $f[0] == 119 )
		{	
			my @len = unpack( 'C', substr( $data, $pos + 1, 1 ) );

			#print "119=>skip $len[0]\n";
			$pos += $len[0];
		}
		# dannie o popolnenii scheta
		elsif( $f[0] == 120 )
		{
			#print "120=>skip\n";
			$pos += 15;
		}
		# disconnect couse
		elsif( $f[0] == 121 )
		{
			#print "121=>skip\n";
			$pos += 5;
		}
		# Charge Band Number
		elsif( $f[0] == 122 )
		{
			#print "122=>skip\n";
			$pos += 5;
		}
		# Common Call Id)
		elsif( $f[0] == 123 )
		{
			#print "123=>skip\n";
			$pos += 6;
		}
		# Durations before Answer
		elsif( $f[0] == 124 )
		{
			#print "124=>skip\n";
			$pos += 10;
		}
		# VoIP Info
		elsif( $f[0] == 125 )
		{
			#print "125=>skip\n";
			$pos += 5;
		}
		# Amount of Transferred Data
		elsif( $f[0] == 126 )
		{			
			#print "126=>skip\n";
			$pos += 13;
		}
		# IP Addresses
		elsif( $f[0] == 127 )
		{
			die "127=>error!!!\n";
		}
		# VoIP Info
		elsif( $f[0] == 128 )
		{
			#print "128=>skip\n";
			$pos += 13;
		}
		# Amount of Transferred Data
		elsif( $f[0] == 129 )
		{
			#print "129=>skip\n";
			$pos += 25;
		}
		else
		{
			die "No processing code $f[0]\n";
		}
    }
    
    return %result;
}

sub get_time
{
	my $param = shift;
	
	# 0 - year (0-99), 1 - month (1-12), 2 - day (1-31), 3 - hour (0-23)
	# 4 - minute (0-59), 5 - second (0-59), 6 - 100 ms (0-9) 
	my @f = unpack( 'CCCCCCC', $param);
	my $result = sprintf( '%02d.%02d.20%02d %02d:%02d:%02d', $f[2], $f[1], $f[0], $f[3], $f[4], $f[5] );
	
	return $result;	
}

sub get_bytes_len
{
    my $tetr_len = shift;
    return $tetr_len % 2 == 0 ? $tetr_len / 2 : $tetr_len / 2 + 1;
}

sub get_duration()
{
	my $start = shift;
	my $stop = shift;
	
	my @fstart = unpack( 'CCCCCCC', $start);
	my @fstop = unpack( 'CCCCCCC', $stop);

	my $d1 = DateTime->new( year   => "20$fstart[0]",
                          month  => $fstart[1],
                          day    => $fstart[2],
                          hour   => $fstart[3],
                          minute => $fstart[4],
                          second => $fstart[5],
                        );

	my $d2 = DateTime->new( year   => "20$fstop[0]",
                          month  => $fstop[1],
                          day    => $fstop[2],
                          hour   => $fstop[3],
                          minute => $fstop[4],
                          second => $fstop[5],
                        );
	
	my $delta = $d2->subtract_datetime_absolute($d1);

	my $result = $delta -> delta_seconds;
	#print "result=$result\n";

	return $result;

}

sub get_bcd
{
    my $param = shift;
    my $len = shift;	
    
         

    my $result = "";
    my $pos = 0;
    my $bpos = 0; 	
    
    for( my $pos = 0; $pos < $len; )
    {
		my $part = substr( $param, $bpos++, 1 );		
		my @f = unpack( 'C', $part);
		
		$result .= ($f[0]&0b11110000)>>4;
		$pos++;
		
		if( $pos < $len )
		{
			$result .= ($f[0]&0b00001111);
			$pos ++;
		}
    }

    return $result;
}	

sub kill_59
{
	my $num = shift;		
	if( $num =~ /^59(\d{5})/ )
  	{
		$num = $1;		
    }
	return $num;
}

close FILE_OUT;

prepare_file( $log_name, $dst_log_path, $DIR );
print "OK\n"